!==============================================================================
!
!  Name   :  CalcCurrentDepFunctsInCell
!
!  Purpose:  CalcCurrentDepFunctsInCell calculates the spatial forest dynamics
!	     on a grid with or without seed dispersal. ContLightDep calculates the
!            continuous light dependence of growth and death.
!
!  Remarks:  contains the subroutines
!
!            CalcCurrentDepFunctsInCell (<DoLocalWork<DistrLocalDynThisTime<
!                                          TimeLoop<TreeMig)
!            ContLightDep (<GetSpc<TreeMig AND/OR <Regen<LocalForestDynamics<
!                            DoLocalWork<DistrLocalDynThisTime<TimeLoop<TreeMig)
!
!==============================================================================
! design          : H. Lischke, N. Zimmermann
! author(s)       : H. Lischke, N. Zimmermann
! implementation  : H. Lischke, N. Zimmermann
! cleaner         : T.J. Loeffler
! copyright       : (c) 1999, 03 by H. Lischke
!==============================================================================

! Jun 2017 - MS:
! LA adapted as species level as a function of drought and temperature stress

SUBROUTINE CalcCurrentDepFunctsInCell(Iam,year, lat, lon,nspc, ddegs, wtemp, drstr, avnit, brwpr,smday)

!<CALL modules for type definitions>---------------------------------------------------------------
USE All_par, ONLY: maxspc, maxhc, maxlc, birht, ienv, kn1, kn2,     &
                   ltval, spc, spec, estabEqHC0, hcwidth, a1, a2, c1
use forhycsglobals, ONLY: curhcmax, &  ! MS - April 2017: Dynamically varying kHmax as a function of drought stress
                          laRed, &     ! MS - Jun 2017: Dynamically varying leaf area as a function of environmental stress
                          dolared,&    ! MS - Jul 2017: Control parameter specifying whether LA should be reduced or not
                          dosmort      ! MS - Jul 2017: Control parameter for snow-induced seedling mortality for Larix and Pinus
IMPLICIT NONE

! <Passed variables>-------------------------------------------------------------------------------
INTEGER,INTENT(IN) :: Iam,year, lat, lon,nspc           ! Passed number of species                             [=spec]
 REAL, INTENT(IN) :: ddegs,        & ! day-degrees of cell                                         [=dayDegs] !ENV
                    drstr,        & ! drough stress of cell                                [=rDroughtStress] !ENV
                    avnit,        & ! av. Nitrogen of cell                                     [=avNitrogen] !ENV
                    wtemp,        & ! winter temperature of cell                               [=winterTemp] !ENV
                    brwpr           ! browsing pressure of cell                                [=browsPress] !ENV
  real, intent(in) :: smday         ! Final day of snow cover

!<Local variables>---------------------------------------------------------------------------------
INTEGER :: i               ! looping variable for species
REAL    :: truht,        & ! tmp var to calculate heights                             [=trueHeight]
           diams,        & ! tmp var to calculate spec. diameters       [in DetermineProc..  =diam]
           alloc1,       & ! tmp allocation var to calculate growth rate                   [=allo1]
           alloc2,       & ! tmp allocation var to calculate growth rate                   [=allo2]
           grofac,       & ! diameter increment growth factor                         [=diamIncFak]
           hcgrow,       & ! height class dep. growth rate                             [=growthInH]
           lih,          & ! local value of ltval(ltcl)                                      [=lih]
           lidep,        & ! light dependent growth                                        [=lidep]

           tdep(maxspc), & ! temperature dependent growth factor                   [=growthTempDep] !ENV
           ddep(maxspc), & ! drought dependent growth factor                    [=growthDroughtDep] !ENV
           ndep(maxspc), & ! nitrogen dependent growth factor                         [=growthNDep] !ENV
           envdpg,       & ! local env. dependent growth factor                      [=allOtherDep] !ENV
           envetb,       & ! temp var. for env. sensitivity to establishment         [=..otheretab] !ENV
           brwetb           ! temp var. for browsing sensit. to establ.               [=..otheretab] !ENV

!<New variables for FORHYCS>------------------------------------------------------------------------

real :: temp_envdpg  ! Just for output
logical :: testfile  ! Just for temp output

real     :: aSnow, bSnow, cSnow, &           ! Parameters for snow cover duration dependence
            mSnow(maxspc), &
            smdayE
integer::           htcl, ltcl
integer :: chcmax       ! Maximum height for each species/cell, as reduced by drought stress (MS-April 2017)
real :: curhmax, curdmax

! Fraction of new dry matter that gets allocated to roots under unstressed conditions
! See Landsberg & Waring (1997), Eq. 13
  real, parameter :: aRns = 0.229
  real :: aSns, aLns    ! C allocation coefficients under unstressed conditions
  real :: aR, aS, aL    ! C allocation coefficients under actual conditions
  real :: pls    ! Variable determining C allocation between foliage and stems
  real :: kb1, kb2, kb3, kl1, kl2, kl3    ! Allometric coefficients for leaf and stem biomass
  real :: dbh

!<Here the subroutine starts>*********************************************************************
  !----- Check whether site data drive dynamics ----------------------------------------------------

   Include_ENV: IF (ienv) THEN
    !----- Environment dependence                                                                                         !
    DO i=1, nspc

    !----- day degree dependence   (saturation function)                                                                                         !
      tdep(i)= MAX(0.0,1.0-exp((1.386/spec(i)%kddmax)*(spec(i)%kddmin-ddegs)) )
     !----- drought dependence
     !----- make drought dependence per cell an array later(!), very important!!
      ddep(i) = (MAX(1.0 - drstr / spec(i)%kdrt, 0.0) ** 0.5)
     !----- nitrogen dependence
      ndep(i)        = MAX (1.0 - EXP (kn1(spec(i)%kntol) * (avnit - kn2(spec(i)%kntol))), 0.0)
      spec(i)%envdep = tdep(i) * ddep(i) * ndep(i)


      ! TEMPORARy - Just for output of relative growth term
      if((lat==172) .and. (lon==72) .and. (i == 6)) then
        temp_envdpg = spec(i)%envdep ** 0.333333
        inquire(file="grow_test.txt",exist=testfile)
        if(testfile) then
          open(5555, file="grow_test.txt",status="old",position="append")
        else
          open(5555, file="grow_test.txt",status="new")
        end if

      ! Year, temperature function, drought function, N function, product, "geometric mean"
56      format(I5,5F7.4,F8.2,F6.2,F8.2)
        write(5555,56) year, tdep(i), ddep(i), ndep(i), spec(i)%envdep, temp_envdpg, ddegs, drstr, avnit
      end if
      ! End temp output

      ! Additional mortality term for Larix and Pinus seedlings - depends on snow cover duration
      if(dosmort == 1) then
      select case (i)
        case(4,5)
          aSnow = 1.38e-04
          bSnow = -3.3e-02
          cSnow = 1.99

          smdayE = max(120.0,min(170.0,smday))

          mSnow(i) = aSnow * smdayE**2 + bSnow * smdayE + cSnow

        case(2)
          aSnow = 6.69e-05
          bSnow = -1.81e-02
          cSnow = 1.24

          smdayE = max(120.0,min(170.0,smday))

          mSnow(i) = aSnow * smdayE**2 + bSnow * smdayE + cSnow

        case default
          mSnow(i) = 0.0

      end select

      spc(i)%snowMortality = mSnow(i)

      else
        ! Control parameter dosmort == 0 => No snow-induced
        msnow(i) = 0.0
        spc(i)%snowMortality = 0.0
      end if

    END DO

    !----- establishment dependence on day degree sum and winter temperature
    Do_Etab: DO i=1,nspc
!    if(year .ge. 171) print*, lat,lon,i,wtemp,spec(i)%kwit,ddegs,spec(i)%kddmin
       Env_Etab: IF  ((wtemp < REAL (spec(i)%kwit  )) .OR. (ddegs < REAL (spec(i)%kddmin)))  THEN
         envetb = 0.0
       ELSE
         envetb = 1.0
       END IF Env_Etab
      !----- establishment dependence on browsing
      brwetb         = 1.0 - (REAL(spec(i)%kbrow - 1) * brwpr / 30.0)
       spec(i)%enveta = envetb * brwetb
    END DO Do_Etab
   ELSE  !----- Include_ENV
    spec%envdep = 1.0
    spec%enveta = 1.0
  END IF Include_ENV
   !-----<Initialize env. dependent tree arrays [=DetermineProcRates]; this has moved from GETSTD>-------
  BySpcs2: DO i=1,nspc
      ! chcmax: Maximum height class that can be reached by Species i in this cell
    ! (calculated in adjhmax). chcmax replaces spec(i)%maxhtc, and curhmax replaces
    ! spec(i)%khmax in the calculations below. curdmax is the DBH corresponding to
    ! curhmax and replaces spec(i)%maxdhm
    ! MS, April 2017


    chcmax = curhcmax(i,lon,lat)
    curhmax = birht + hcwidth*(chcmax-1)
    curdmax = spc(i)%diain(chcmax)

     ByHtCls2: DO htcl=spec(i)%maxhtc,1,-1
      envdpg = spec(i)%envdep ** 0.333333
      diams  = spc(i)%diain(htcl)
      truht  = spc(i)%htmin(htcl)
      !alloc1 = (2.0 * (REAL(spec(i)%khmax) - birht) / spec(i)%maxdhm)
      alloc1 = (2.0 * (curhmax - birht) / curdmax)
      !alloc2 = -alloc1 / 2.0 / spec(i)%maxdhm
      alloc2 = -alloc1 / 2.0 / curdmax
      !IF (htcl <= spec(i)%maxhtc) THEN
      IF (htcl <= chcmax) THEN
        !grofac = (envdpg * spec(i)%maxgro * diams * (1.0 - (truht / REAL(spec(i)%khmax)))) / &
        !         (2.74 + (((3.0 * alloc1) + (4.0 * alloc2 * diams)) * diams))
        grofac = (envdpg * spec(i)%maxgro * diams * (1.0 - (truht / curhmax))) / &
                 (2.74 + (((3.0 * alloc1) + (4.0 * alloc2 * diams)) * diams))
      ELSE
        grofac = 0.0
      END IF

      hcgrow = (grofac * (alloc1 + (2.0 * alloc2 * diams)))

      !----- light dependence values in light classes of growth and mortality
      ByLtCls2: DO ltcl=1,maxlc
        lih = ltval(ltcl)

        !----- THIS STEP PRODUCES MOST OF THE COMPUTING TIME
!----->OLD        CALL ContLightDep (lih, spec(i)%kligha, lidep)
        lidep = spc(i)%lightDepA(ltcl)

        spc(i)%hlgro(htcl,ltcl) = lidep * hcgrow
        spc(i)%envVitality(htcl,ltcl) = lidep * envdpg
        spc(i)%growthVitality(htcl,ltcl) = lidep * grofac

      END DO ByLtCls2

        ! ---------- Leaf area reduction - MS, June 2017 -------------------



        if(dolared == 1) then
          ! Allometric parameters for stem and leaf biomass
          kb1 = 0.5
          kb2 = 0.12
          kb3 = 2.4
          kl1 = c1(spec(i)%tb)
          kl2 = a1(spec(i)%stn)
          kl3 = a2(spec(i)%stn) ! previously wrong (was a1 instead of a2)
          dbh = spc(i)%diain(htcl)*100

          ! Allocation coefficients for stem and foliage under unstressed conditions
          ! aRns: set to 0.229
          pls = (kl1 * kl2 * kl3 * dbh**(kl3-1))/(kb1 * kb2 * kb3 * dbh**(kb3-1))
          aSns = (1-aRns)/(pls + 1)
          aLns = 1 - aSns - aRns

          ! Under stressed conditions:
          aR = 0.8/(1+2.5*(1-envdpg))
          aS = (1-aR)/(pls+1)
          aL = 1 - aS - aR

          laRed(i,htcl) = min(1.0, aL/aLns)

        else
        ! No leaf area reduction (if control parameter dolared == 0)
          laRed(i,htcl) = 1.0
        end if


    END DO ByHtCls2

    !----- function values of 0th height class (seedlings to 1.37 m)

    ByLtCls4: DO ltcl=1,maxlc
      IF (estabEqHC0) THEN
        !------ if wished, light dependence of establishment (from FORCLIM) is contributed to 0th height class
        lih = ltval(ltcl)

        !----- THIS STEP PRODUCES MOST OF THE COMPUTING TIME
!----->OLD        CALL ContLightDep (lih, spec(i)%kligha, lidep)
        lidep=spc(i)%lightDepE(ltcl)

        spc(i)%hlgro(0,ltcl) = lidep * hcgrow
        spc(i)%envVitality(0,ltcl) = lidep * envdpg
        spc(i)%growthVitality(0,ltcl) = lidep * grofac


      ELSE
        !----- otherwise the values for the first height class are also used for the 0th one (seedlings)
        spc(i)%hlgro(0,ltcl) = spc(i)%hlgro(1,ltcl)
        spc(i)%envVitality(0,ltcl) = spc(i)%envVitality(1,ltcl)
        spc(i)%growthVitality(0,ltcl) = spc(i)%growthVitality(1,ltcl)
      END IF
    END DO ByLtCls4

    !IF (spec(i)%maxhtc < maxhc) THEN
    IF (chcmax < maxhc) THEN
      !RestHtCls2: DO htcl = spec(i)%maxhtc+1, maxhc
      RestHtCls2: DO htcl = chcmax+1, maxhc
        ByLtCls3: DO ltcl = 1 , maxlc
          spc(i)%hlgro(htcl,ltcl) = 0.0
          spc(i)%envVitality(htcl,ltcl) = 0.0
          spc(i)%growthVitality(htcl,ltcl) = 0.0
        END DO ByLtCls3
      END DO RestHtCls2
    END IF
   END DO BySpcs2
END SUBROUTINE CalcCurrentDepFunctsInCell
!****<Here the subroutine ends>********************************************************************



!****<Here the subroutine ContLightDep starts>*****************************************************
SUBROUTINE ContLightDep(light, threshold, lidep)

!<CALL modules for type definitions>---------------------------------------------------------------
IMPLICIT NONE

!<Passed variables>--------------------------------------------------------------------------------
INTEGER, INTENT(IN) :: threshold    ! Spec. specific light threshold
REAL, INTENT(IN)    :: light        ! Light intensity
REAL, INTENT(OUT)   :: lidep        ! Light dependence

!<Local variables>---------------------------------------------------------------------------------
REAL :: gl1, gl1old,          & ! light response for shade-tolerant species                  [=gl1]
        gl9, gl9old,          & ! light response for shade-intolerant species                [=gl9]
        eps, steepness,       &
        contstep1, contstep9

!<Here the subroutine starts>*********************************************************************
  eps       = 0.00000001
  steepness = 5
  contstep1 = eps * (1 - EXP (-((light / 0.05) ** steepness)))
  contstep9 = eps * (1 - EXP (-((light / 0.08) ** steepness)))
  gl1old    = 1.0 - EXP (-4.64 * (light - 0.05))
  gl1       = MAX (gl1old + eps, 0.0) + contstep1
  gl9old    = 2.24 * (1.0 - EXP (-1.136 * (light - 0.08)))
  gl9       = MAX (gl9old + eps, 0.0) + contstep9
  lidep     = (gl1 + ((threshold - 1.0) * (gl9 - gl1) / 8.0))

  if ((lidep < 0.0) .OR. (lidep > 2)) THEN
    WRITE(1070,*) 'neg lidep', light, contstep1, contstep9, gl1old, gl9old, gl1, gl9, threshold, lidep
  END IF

  lidep = MAX (lidep, 0.0) ** 0.333333
END SUBROUTINE ContLightDep
!****<Here the subroutine ends>********************************************************************
